﻿using Statki.Model;
using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Statki
{
    public class BoardGUI
    {
        private const int leftMarginOfMap = 10;

        //margines mapy od góry
        public int TopMarginOfMap { private set; get; }
        //rozmiar przycisku
        public int ButtonSize { private set; get; }

        Gameplay Gameplay;
        AI AI;

        FieldClickCheckerPlanning FieldClickCheckerPlanning;
        public Dictionary<Field, Button> FieldToButtonMapping { get; private set; }

        private Button[,] boardForFirstPlayer;
        private Button[,] boardForSecondPlayer;
        private Field[,] MatrixForAI;

        public event EventHandler OnBoardFieldClicked;
        
        public int Height { get; private set; }
        public int Width { get; private set; }

        /// <summary>
        /// Konstruktor klasy Board, odpowiada za utworzenie planszy gry
        /// </summary>
        /// <param name="Height">Wysokość ekranu</param>
        /// <param name="Width">Szerokość ekranu</param>
        public BoardGUI(int Height, int Width, Gameplay gameplay, FieldClickCheckerPlanning fieldClickCheckerPlanning)
        {
            FieldToButtonMapping = new Dictionary<Field, Button>();
            FieldClickCheckerPlanning = fieldClickCheckerPlanning;
            this.Height = Height;
            this.Width = Width;
            Gameplay = gameplay;
            TopMarginOfMap = Height / 40;
            ButtonSize = (Math.Min(Width, Height) * 2) / (gameplay.BoardDimension * 3);
        }

        /// <summary>
        /// Tworzy mapę gry
        /// </summary>
        /// <param name="parent">Okienko do rysowania</param>
        public void CreateMaps(Control parent)
        {
            boardForFirstPlayer = new Button[Gameplay.BoardDimension + 1, Gameplay.BoardDimension + 1];
            boardForSecondPlayer = new Button[Gameplay.BoardDimension + 1, Gameplay.BoardDimension + 1];

            for (int x = 0; x <= Gameplay.BoardDimension; x++)
            {
                for (int y = 0; y <= Gameplay.BoardDimension; y++)
                {
                    boardForFirstPlayer[x, y] = CreateField(Gameplay.FieldsPlayer1[x, y], ButtonSize, parent, PlayerEnum.PlayerOne);
                    boardForSecondPlayer[x, y] = CreateField(Gameplay.FieldsPlayer2[x, y], ButtonSize, parent, PlayerEnum.PlayerTwo);
                    boardForSecondPlayer[x, y].Enabled = false;
                }
            }

            MarkBasicFields();

            PlaceFieldsProperly();
        }
        
        /// <summary>
        /// Tworzy legende dla planszy
        /// </summary>
        private void MarkBasicFields()
        {
            boardForFirstPlayer[0, 0].BackColor = Color.Transparent;
            boardForSecondPlayer[0, 0].BackColor = Color.Transparent;

            for (int x = 1; x <= Gameplay.BoardDimension; x++)
            {
                boardForFirstPlayer[x, 0].Text = ((char)(x + 64)).ToString();
                boardForFirstPlayer[x, 0].BackColor = Color.Gray;
                boardForSecondPlayer[x, 0].Text = ((char)(x + 64)).ToString();
                boardForSecondPlayer[x, 0].BackColor = Color.Gray;
            }

            for (int y = 1; y <= Gameplay.BoardDimension; y++)
            {
                boardForFirstPlayer[0, y].Text = y.ToString();
                boardForFirstPlayer[0, y].BackColor = Color.Gray;
                boardForSecondPlayer[0, y].Text = y.ToString();
                boardForSecondPlayer[0, y].BackColor = Color.Gray;
            }
        }

        /// <summary>
        /// Umieść pola zależnie od rozmiaru okienka
        /// </summary>
        private void PlaceFieldsProperly()
        {
            //Marginesy i rozmieszczenie
            for (int x = 0; x <= Gameplay.BoardDimension; x++)
            {
                for (int y = 0; y <= Gameplay.BoardDimension; y++)
                {
                    boardForFirstPlayer[x, y].Left = x * ButtonSize;
                    boardForFirstPlayer[x, y].Left += leftMarginOfMap;
                    boardForFirstPlayer[x, y].Top = TopMarginOfMap + y * ButtonSize;

                    boardForSecondPlayer[x, y].Left = (x + Gameplay.BoardDimension) * ButtonSize;
                    boardForSecondPlayer[x, y].Left += leftMarginOfMap * 5;
                    boardForSecondPlayer[x, y].Top = TopMarginOfMap + y * ButtonSize;
                }
            }
        }

        /// <summary>
        /// Tworzy pole planszy
        /// </summary>
        /// <param name="x">oś x</param>
        /// <param name="y">oś y</param>
        /// <param name="ButtonSide">Rozmiar przycisku</param>
        /// <param name="parent">Okienko</param>
        /// <returns></returns>
        private Button CreateField(Field field, int ButtonSide, Control parent, PlayerEnum gracz)
        {
            Button b = new Button
           
            {
                //Rozmiar przycisków
                Width = ButtonSide,
                Height = ButtonSide,

                Parent = parent,
                BackColor = Color.Aqua
            };
            if (field.Coordinate.X != 0 && field.Coordinate.Y != 0)
                b.Click += (sender, e) => Do_Action_On_Button_Click(sender, e, Gameplay.GamePhase);

            FieldToButtonMapping.Add(field, b);

            b.Tag = new Tuple<Field, PlayerEnum>(field, gracz);

            return b;
        }
        
        /// <summary>
        /// Tu się będzie działo!
        /// </summary>
        /// <param name="sender">Który to?!</param>
        /// <param name="e">Coś narobił?!</param>
        private void Do_Action_On_Button_Click(object sender, EventArgs e, GamePhase gamePhase)
        {
            switch (gamePhase)
            {
                case GamePhase.PlanningPhase:
                    Button b = sender as Button;
                    Field clickedField = (b.Tag as Tuple<Field, PlayerEnum>).Item1;
                    if (!FieldClickCheckerPlanning.CheckIfFieldContainShip(clickedField))
                    {
                        FieldClickCheckerPlanning.AddFieldToList(clickedField);
                        b.BackColor = Color.Green;
                    }
                    else
                    {
                        FieldClickCheckerPlanning.RemoveFieldToList(clickedField);
                        b.BackColor = Color.Aqua;
                    }

                    OnBoardFieldClicked?.Invoke(this, EventArgs.Empty);
                    break;
                case GamePhase.ShootingPhase:
                    ShootingScenario(sender, e);
                    break;
                default:
                    break;
            }
        }
        private void BlockFields(Field field)
        {
            for(int i = -1; i <= 1; i++)
            {
                for(int j = -1; j <= 1; j++)
                {
                    //sprawdzanie czy nie wychodzimy poza zakres planszy, następnie blokujemy buttony
                    if(field.Coordinate.X + i <= Gameplay.BoardDimension && field.Coordinate.Y + j <= Gameplay.BoardDimension)
                        boardForFirstPlayer[field.Coordinate.X + i, field.Coordinate.Y + j].Enabled = false;  
                }
            }

        }
        public void LockFields()
        {

            foreach (Field field in FieldClickCheckerPlanning.ClickedFields)
            {
                FieldToButtonMapping.TryGetValue(field, out Button b);
                if (b != null)
                {
                    Block_Field_With_Its_Surrounding(field);
                }
            }

            FieldClickCheckerPlanning.ClickedFields.Clear();
        }

        private void Block_Field_With_Its_Surrounding(Field field)
        {
            for (int i = -1; i <= 1; i++)
            {
                for (int j = -1; j <= 1; j++)
                {
                    //sprawdzanie czy nie wychodzimy poza zakres planszy, następnie blokujemy buttony
                    if (field.Coordinate.X + i <= Gameplay.BoardDimension && field.Coordinate.Y + j <= Gameplay.BoardDimension)
                        boardForFirstPlayer[field.Coordinate.X + i, field.Coordinate.Y + j].Enabled = false;
                }
            }

        }
        
        //Poniższe metody do fazy gry strzelanie

        public void StartGame()
        {
            ChangeEnablelityOfButtons();
            Create_Matrix_For_AI();
        }

        public void ChangeEnablelityOfButtons()
        {
            for(int i = 1; i <= Gameplay.BoardDimension; i++)
                for(int j = 1; j <= Gameplay.BoardDimension; j++)
                {
                    boardForFirstPlayer[i, j].Enabled = false;
                    boardForSecondPlayer[i, j].Enabled = true;
                }
        }
        
        private void Create_Matrix_For_AI()
        {
            AI = new AI();

            MatrixForAI = new Field[Gameplay.BoardDimension + 1, Gameplay.BoardDimension + 1];
            for(int i = 0; i <= Gameplay.BoardDimension; i++)
            {
                MatrixForAI[0, i] = new Field(new Coordinate(0, i), FieldState.Misshit);
                MatrixForAI[i, 0] = new Field(new Coordinate(i, 0), FieldState.Misshit);
            }

            foreach (KeyValuePair<Field, Button> fieldToButton in FieldToButtonMapping)
            {
                if (((fieldToButton.Value.Tag) as Tuple<Field, PlayerEnum>).Item2 == PlayerEnum.PlayerOne
                    && fieldToButton.Key.Coordinate.X != 0
                    && fieldToButton.Key.Coordinate.Y != 0)
                {
                    MatrixForAI[fieldToButton.Key.Coordinate.X, fieldToButton.Key.Coordinate.Y] = fieldToButton.Key;
                }
            }
        }

        private void ShootingScenario(object sender, EventArgs e)
        {
            Tuple<Field, PlayerEnum> field = (sender as Button).Tag as Tuple<Field, PlayerEnum>;
            
            if (field.Item1.Ship != null)
                boardForSecondPlayer[field.Item1.Coordinate.X, field.Item1.Coordinate.Y].BackColor = Color.Red;

            boardForSecondPlayer[field.Item1.Coordinate.X, field.Item1.Coordinate.Y].Text = "X";
            if (field.Item1.Ship != null && field.Item1.Ship.Sunk())
            {
                foreach(Field fieldd in field.Item1.Ship.Fields)
                {
                    boardForSecondPlayer[fieldd.Coordinate.X, fieldd.Coordinate.Y].BackColor = Color.Black;
                }
                
            }
            Gameplay.UpdateTurn(field.Item2);
            Check_If_Player_Wins();
            AI_makes_shot();

        }

        private void AI_makes_shot()
        {
            Coordinate shotCoordinates = AI.GetNextShoot(MatrixForAI);

            FieldToButtonMapping.TryGetValue(MatrixForAI[shotCoordinates.X, shotCoordinates.Y], out Button b);

            if (b != null)
                b.Text = "X";

            if (MatrixForAI[shotCoordinates.X, shotCoordinates.Y].Ship != null)
            {
                if (b != null)
                    b.BackColor = Color.Red;
            }

            Gameplay.UpdateField(MatrixForAI[shotCoordinates.X, shotCoordinates.Y], PlayerEnum.PlayerTwo);
            Check_If_AI_Wins();
        }

        private void Check_If_Player_Wins()
        {
            if (Gameplay.ShipCollectionPlayer2.All(ship => ship.IsSunk == true))
            {
                Gameplay.LaunchAnotherPhase();
                LaunchSummaryPhase(PlayerEnum.PlayerOne);

                for (int i = 1; i <= Gameplay.BoardDimension; i++)
                    for (int j = 1; j <= Gameplay.BoardDimension; j++)
                        boardForSecondPlayer[i, j].Enabled = false;
            }
        }

        private void Check_If_AI_Wins()
        {
            if (Gameplay.ShipCollectionPlayer1.All(ship => ship.IsSunk == true))
            {
                Gameplay.LaunchAnotherPhase();
                LaunchSummaryPhase(PlayerEnum.PlayerTwo);

                for (int i = 1; i <= Gameplay.BoardDimension; i++)
                    for (int j = 1; j <= Gameplay.BoardDimension; j++)
                        boardForSecondPlayer[i, j].Enabled = false;
            }
        }

        /// <summary>
        /// Metoda jest wywoływana w momencie gdy gracz lub komputer zestrzeli ostatni statek przeciwnika
        /// </summary>
        /// <param name="winner">Napewno nie ten, któremu zatonęły wszystkie statki</param>
        public void LaunchSummaryPhase(PlayerEnum winner)
        {
            string message = string.Format("Congratz! {0} wins", winner);

            MessageBox.Show(message, "Game has ended", MessageBoxButtons.OK, MessageBoxIcon.Asterisk); // Obelix i Idefix

            if (winner == PlayerEnum.PlayerTwo)
            {
                foreach (KeyValuePair<Field, Button> fieldToButton in FieldToButtonMapping)
                {
                    if (fieldToButton.Key.Ship != null
                        && (fieldToButton.Value.Tag as Tuple<Field, PlayerEnum>).Item2 == PlayerEnum.PlayerTwo
                        && fieldToButton.Key.FieldState == FieldState.Blank)
                    {
                        fieldToButton.Value.BackColor = Color.Blue;
                    }
                }
            }
        }
    }
}
